/* FRAME -- by Mark Cederholm

 * Reads the .FGD file of an USGS DRG image
   and writes an Idrisi vector file representing the map border in DD

 * Version 1.0, 3/25/98

 * Usage:  FRAME [infile] [outfile]

 * Arguments:   infile = input metadata file (.FGD extension assumed)
               outfile = output vector file (no extension)

/*********************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char *argv[])
{

   /**** initialize environment **************************************/

   FILE *infile, *vecfile, *docfile;
   char fgd[81];         /* .FGD file name                           */
   char vec[81];         /* .VEC file name                           */
   char doc[81];         /* .DOC file name                           */
   char input[81];       /* input line                               */
   char temp[81];        /* working string                           */
   char *tok;            /* current token                            */
   char west[20];        /* quad west coordinate                     */
   char east[20];        /* quad east coordinate                     */
   char north[20];       /* quad north coordinate                    */
   char south[20];       /* quad south coordinate                    */
   double yshift;        /* y-shift                                  */
   int tmp;              /* temp variable                            */

   /* check # of arguments */

   if ((argc != 3) && (argc != 4))
   {
      printf("Usage:  FRAME [fgd_file] [vec_file] \n");
      return 0;
   };

   /* initialize variables */

   strcpy(fgd, argv[1]);
   strcpy(vec, argv[2]);
   strcpy(doc, argv[2]);
   strcat(fgd, ".fgd");
   strcat(vec, ".vec");
   strcat(doc, ".dvc");

   /* open files */

   if ((infile = fopen(fgd,"r")) == NULL)
   {
      printf("Error opening input file %s.\n", fgd);
      return 0;
   };
   if ((vecfile = fopen(vec,"w")) == NULL)
   {
      printf("Error opening output file %s.\n", vec);
      return 0;
   };
   if ((docfile = fopen(doc,"w")) == NULL)
   {
      printf("Error opening output file %s.\n", doc);
      return 0;
   };

   /* read info from FGD file */

   while (fgets(input, 80, infile) != NULL)
   {
      strcpy(temp,"1.5.1.1  WEST BOUNDING COORDINATE:");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         strcpy(west,tok);
         continue;
      }

      strcpy(temp,"1.5.1.2  EAST BOUNDING COORDINATE:");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         strcpy(east,tok);
         continue;
      }

      strcpy(temp,"1.5.1.3  NORTH BOUNDING COORDINATE:");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         strcpy(north,tok);
         continue;
      }

      strcpy(temp,"1.5.1.4  SOUTH BOUNDING COORDINATE:");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         strcpy(south,tok);
         continue;
      }

   }

   /* write VEC file */

   fprintf(vecfile,"1 5\n");
   fprintf(vecfile,"%s %s\n", west, south);
   fprintf(vecfile,"%s %s\n", west, north);
   fprintf(vecfile,"%s %s\n", east, north);
   fprintf(vecfile,"%s %s\n", east, south);
   fprintf(vecfile,"%s %s\n", west, south);
   fprintf(vecfile,"0 0\n");

   /* write DOC file */

   fprintf(docfile,"file title  : Frame file\n");
   fprintf(docfile,"id type     : integer\n");
   fprintf(docfile,"file type   : ascii\n");
   fprintf(docfile,"object type : line\n");
   fprintf(docfile,"ref. system : latlong\n");
   fprintf(docfile,"ref. units  : deg\n");
   fprintf(docfile,"unit dist.  : 1.0000000\n");
   fprintf(docfile,"min. X      : %s\n", west);
   fprintf(docfile,"max. X      : %s\n", east);
   fprintf(docfile,"min. Y      : %s\n", south);
   fprintf(docfile,"max. Y      : %s\n", north);
   fprintf(docfile,"pos'n error : unknown\n");
   fprintf(docfile,"resolution  : unknown\n");

   return 0;

}

