/* REDOCW -- by Mark Cederholm

 * Reads the .TFW file of an USGS DRG image
   and updates the .DOC file accordingly

 * Version 1.0, 8/18/98

 * Usage:  REDOC [infile] [outfile] {y-shift}

 * Arguments:   infile = input metadata file (.TFW extension assumed)
               outfile = output documentation file (.DOC extension assumed)
               y-shift = optional y-shift to be applied to coordinates

/*********************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char *argv[])
{

   /**** initialize environment **************************************/

   FILE *infile, *outfile, *bakfile;
   char tfw[81];         /* .TFW file name                           */
   char doc[81];         /* .DOC file name                           */
   char bak[81];         /* .BAK file name                           */
   char input[81];       /* input line                               */
   char temp[81];        /* working string                           */
   char *tok;            /* current token                            */
   double yshift;        /* y-shift                                  */
   double x, y;          /* base coordinate                          */
   double rez;           /* cell size                                */
   double val;           /* output value                             */
   int row, col;         /* num of rows and columns                  */
   int i, j, tmp;        /* temp variables                           */

   /* check # of arguments */

   if ((argc != 3) && (argc != 4))
   {
      printf("Usage:  REDOC [tfw_file] [doc_file] {y-shift}\n");
      return 0;
   };

   /* initialize variables */

   strcpy(tfw, argv[1]);
   strcpy(doc, argv[2]);
   strcpy(bak, argv[2]);
   strcat(tfw, ".tfw");
   strcat(doc, ".doc");
   strcat(bak, ".bak");

   if (argc == 3)
      yshift = 0.0;
   else
      yshift = atof(argv[3]);

   /* rename .DOC file to .BAK */

   if (rename(doc, bak) != 0)
   {
      printf("Error:  could not rename %s to %s.\n", doc, bak);
      return 0;
   }

   /* open files */

   if ((infile = fopen(tfw,"r")) == NULL)
   {
      printf("Error opening input file %s.\n", tfw);
      return 0;
   };
   if ((bakfile = fopen(bak,"r")) == NULL)
   {
      printf("Error opening input file %s.\n", bak);
      return 0;
   };
   if ((outfile = fopen(doc,"w")) == NULL)
   {
      printf("Error opening output file %s.\n", doc);
      return 0;
   };

   /* read info from TFW file */

   fgets(input, 80, infile);
   rez = atof(input);
   fgets(input, 80, infile);
   fgets(input, 80, infile);
   fgets(input, 80, infile);
   fgets(input, 80, infile);
   x = atof(input);
   fgets(input, 80, infile);
   y = atof(input);

   /* process DOC file */

   while (fgets(input, 80, bakfile) != NULL)
   {

      strcpy(temp,"columns");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         col = atoi(tok);
         fprintf(outfile,"%s", input);
         continue;
      }

      strcpy(temp,"rows");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         strcpy(temp,input);
         tok = strtok(temp, " \n");
         tok = strtok(NULL, " \n");
         tok = strtok(NULL, " \n");
         row = atoi(tok);
         fprintf(outfile,"%s", input);
         continue;
      }

      strcpy(temp,"min. X");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         val = x - ( rez / 2 );
         fprintf(outfile,"min. X      : %f\n", val);
         continue;
      }

      strcpy(temp,"max. X");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         val = x - ( rez / 2 ) + ( col * rez );
         fprintf(outfile,"max. X      : %f\n", val);
         continue;
      }

      strcpy(temp,"min. Y");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         val = y + ( rez / 2 ) - ( row * rez ) + yshift;
         fprintf(outfile,"min. Y      : %f\n", val);
         continue;
      }

      strcpy(temp,"max. Y");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         val = y + ( rez / 2 ) + yshift;
         fprintf(outfile,"max. Y      : %f\n", val);
         continue;
      }

      strcpy(temp,"resolution");
      tmp = strlen(temp);
      if (strncmp(input,temp,tmp) == 0)
      {
         fprintf(outfile,"resolution  : %f\n", rez);
         continue;
      }

      fprintf(outfile,"%s", input);
   }

   /* delete .BAK file */

   if (remove(bak) != 0)
   {
      printf("Error:  could not delete %s.\n", bak);
      return 0;
   }

   return 0;

}

